package com.belkast.soap;

import com.beust.jcommander.ParameterException;
import com.beust.jcommander.*;

import java.io.File;
import java.net.HttpURLConnection;
import java.text.SimpleDateFormat;
import java.util.*;

public class webService {
    static Date dateStart = new Date();
    static SimpleDateFormat varFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");

    static Boolean varDebug = false;
    static String varKey;

    public static void main(String... argv) throws Exception {

//        @Parameter(names = "--props",
//          description = "Location of the Properties file")

//        @Parameter(names = "--key",
//          description = "Encryption Key (length of 16)")

//        @Parameter(names = "--encrypt",
//          description = "Encrypt this value using the Key")

//        @Parameter(names = "--debug",
//          description = "Write details to the screen")

        jCommanderArgs args = new jCommanderArgs();
        JCommander jct = new JCommander(args);

        try {
            jct.parse(argv);
        } catch (ParameterException ex) {
            System.out.println();
            jct.usage();
            System.exit(0);
        }

        System.out.println();

        varDebug = Boolean.valueOf(args.debug);
        varKey = args.key;
        String varToEncrypt = args.encrypt;

        if (varToEncrypt != null && !varToEncrypt.isEmpty())
        {
            String varToStore = secretProcessor.encode(secretProcessor.encrypt(varKey, varToEncrypt));
            String varChecked = secretProcessor.decrypt(varKey, secretProcessor.decode(varToStore));
            System.out.println("Clear Text Password : " + varToEncrypt);
            System.out.println("Encryption Key      : " + varKey);
            System.out.println("Encrypted / Encoded : " + varToStore);
            System.out.println("Decoded / Decrypted : " + varChecked);
            System.out.println();
            System.exit(0);
        }

        String varProps = args.props;
        if (varProps == null && varToEncrypt == null)
        {
            debugProcessor.writer(true, "Check the program parameters. There is nothing to do!");
            System.out.println();
            jct.usage();
            System.exit(0);
        }

        if (!fileProcessor.getFile(varProps))
        {
            System.exit(0);
        }

        doSoap(new File(varProps));
        System.exit(0);
    }

    private static String doSoap(File thisProps) throws Exception {
        String varPassword = new String();
        String varDelimiter = propertyProcessor.getProperty(thisProps, "ARGUMENT_DELIMITER");

        debugProcessor.writer(true, "============ START ============");

        debugProcessor.writer(true, "Start Time: " + varFormat.format(dateStart));
        debugProcessor.writer(varDebug, "Debug is set to : " + varDebug);

        System.out.println();
        String varURL = propertyProcessor.getProperty(thisProps, "SHIM_URL");
        String varRoleFile = propertyProcessor.getProperty(thisProps, "INPUT_FILE");
        String varInputFile = propertyProcessor.getProperty(thisProps, "XML_FILE");
        String varUsername = propertyProcessor.getProperty(thisProps, "USERNAME");
        String varPasswordTemp = propertyProcessor.getProperty(thisProps, "PASSWORD");

        String varUseSSL = propertyProcessor.getProperty(thisProps, "SSL");

        String varKeystore_LOC = propertyProcessor.getProperty(thisProps, "JAVA_KS_LOCATION");
        String varKeystore_PW = propertyProcessor.getProperty(thisProps, "JAVA_KS_PASSWORD");

        debugProcessor.writer(varDebug, thisProps + " => SOAP URL : " + varURL);
        debugProcessor.writer(varDebug, thisProps + " => Username : " + varUsername);
        debugProcessor.writer(varDebug, thisProps + " => Use SSL : " + varUseSSL);
        debugProcessor.writer(varDebug, thisProps + " => JAVA Keystore : " + varKeystore_LOC);
        debugProcessor.writer(varDebug, thisProps + " => JAVA Keystore password : " + varKeystore_PW);
        debugProcessor.writer(varDebug, thisProps + " => Input File : " + varRoleFile);
        debugProcessor.writer(varDebug, thisProps + " => XML File : " + varInputFile);

        File checkInput = new File (varInputFile);
        if (!checkInput.exists())
        {
            debugProcessor.writer(varDebug, "The XML_FILE file specified in " + thisProps + " does not exist!");
            System.exit(0);
        }

        File checkRole = new File (varRoleFile);
        if (!checkRole.exists())
            {
                debugProcessor.writer(varDebug, "The INPUT_FILE specified in " + thisProps + " does not exist!");
                System.exit(0);
            }

        try {
            byte[] varDecoded = secretProcessor.decode(varPasswordTemp);
            varPassword = secretProcessor.decrypt(varKey, varDecoded);
        } catch (Exception e) {
            debugProcessor.writer(true, "The password could not be decrypted. Please check that the Encryption Key is correct!");
            System.exit(0);
        }

        ArrayList thisCSV = userVerify.readCSV(varRoleFile, false);

        boolean validfile = (thisCSV.size() > 0);
        debugProcessor.writer(true, varRoleFile + " : CSV file is valid  => " + validfile);
        debugProcessor.writer(true, varRoleFile + " : records to process => " + thisCSV.size());

        if (!validfile)
        {
            debugProcessor.writer(true, varRoleFile + " : verify the file with com.belkast.soap.userVerify");
        }
        int varCount = 0;
        Iterator<Map> mapper1 = thisCSV.iterator();
        while (mapper1.hasNext()) {
            varCount++;
            debugProcessor.writer(false, "");
            debugProcessor.writer(true, "Processing record " + varCount);

            Map hashMap = mapper1.next();
            debugProcessor.writer(varDebug, "Record " + varCount + " : " + hashMap.values().toString());

            Iterator<Map.Entry<Integer, Integer>> it = hashMap.entrySet().iterator();
            while (it.hasNext())
            {
                Map.Entry<Integer, Integer> pair = it.next();
                debugProcessor.writer(varDebug, "Record " + varCount + " : " + pair.getKey() + " => " + pair.getValue());
            }

            String varDocument = fileProcessor.replaceInFile(varInputFile, hashMap);

            if (varDocument.length() > 0) ;
            {
                String varCredentials = soapProcessor.getCredentials(varUsername, varPassword);
                Boolean isSSL = Boolean.parseBoolean(varUseSSL);

                debugProcessor.writer(varDebug, "Connecting to URL : " + varURL);
                HttpURLConnection varConn = soapProcessor.getConnection(varCredentials, varURL, isSSL, varKeystore_LOC, varKeystore_PW);

                debugProcessor.writer(varDebug, "Sending document : " + varDocument);
                String response = soapProcessor.send(varConn, varInputFile, hashMap, varDocument.getBytes());

                debugProcessor.writer(varDebug, "Response received : " + response);
                soapProcessor.closeConnection(varConn);
            }
        }
        Date dateEnd = new Date();
        System.out.println();
        debugProcessor.writer(true, "Finish Time: " + varFormat.format(dateEnd));
        debugProcessor.writer(true, "============ FINISH ============");
        return new String();
    }
}